/***************************************************************************************************
* Author: 
*        Victorino Sanz 
*        Dpto. Informática y Automática, UNED
*        Juan del Rosal, 16
*        28040, Madrid
*        Spain
*        Email: vsanz@dia.uned.es
*
* Licensed by Victorino Sanz under the LGPL License
* Copyright © 2013, Victorino Sanz.
* 
****************************************************************************************************/

#ifndef CELL1D
#define CELL1D



#include <stdio.h>
#include <stdlib.h>




/* Object data type */

typedef struct Cell{
    int phase;
    int x;
    int next;
    int prev;
} Cell;

typedef struct CellSpace{
    Cell *M;
    int n;
}CellSpace;




/* Object list functions ###################################################33*/
CellSpace CellSpaceCreate(int size){
    int i;
    CellSpace s;
    
    s.M = (Cell *)malloc(size*sizeof(Cell));
    s.n = size;
    for(i=0;i<size;i++){
	s.M[i].phase = 0; // start dead
	s.M[i].x = i; // position in the space
	s.M[i].next = 0; // next neighbor dead
	s.M[i].prev = 0; // prev neibhbor dead
    }
    
    return s;   
}

int CellSpaceDelete(CellSpace s){
    free(s.M);
    s.n = 0;
    return 1;
}

int CellSpaceAdd(CellSpace *s, int pos, int newphase){
    int i;

    s->M[pos].phase = newphase;
    // update neighbors (wrapped boundaries)
    if (pos ==  0){
	s->M[pos+1].prev = newphase; 
	s->M[s->n].next = newphase;
    }else if (pos == s->n){
	s->M[0].prev = newphase;
	s->M[pos-1].next = newphase;
    }else{
	s->M[pos+1].prev = newphase;
	s->M[pos-1].next = newphase;
    }
	
    for(i=0;i<s->n;i++){
	if (s->M[i].phase == 0)
	    printf("_");
	else
	    printf("X");
    }
    printf("\n");
    return 1;
}


int CellSpaceStep(CellSpace *s, int (*rule)(int,int,int)){
    int new[s->n];
    int i;

    // calculate and print new phases
    for(i=0;i<s->n;i++){
	new[i] = rule(s->M[i].phase,s->M[i].next,s->M[i].prev);
	if (new[i] == 0)
	    printf("_");
	else
	    printf("X");
    }
    printf("\n");

    // update phases
    for(i=0;i<s->n;i++)
	s->M[i].phase = new[i];
    
    // update neighbor states
    s->M[0].prev = new[(s->n)-1];
    for(i=1;i<s->n;i++)
	s->M[i].prev = new[i-1];
    s->M[s->n].next = new[0];
    for(i=0;i<(s->n)-1;i++)
	s->M[i].next = new[i+1];

    return 1;
}

// RULE FUNCTIONS

int rule30(int phase, int next, int prev){
    int out;

    if(      prev == 1 && phase == 1 && next == 1){
	out = 0;
    }else if(prev == 1 && phase == 1 && next == 0){
	out = 0;
    }else if(prev == 1 && phase == 0 && next == 1){
	out = 0;
    }else if(prev == 1 && phase == 0 && next == 0){
	out = 1;
    }else if(prev == 0 && phase == 1 && next == 1){
	out = 1;
    }else if(prev == 0 && phase == 1 && next == 0){
	out = 1;
    }else if(prev == 0 && phase == 0 && next == 1){
	out = 1;
    }else if(prev == 0 && phase == 0 && next == 0){
	out = 0;
    }

    return out;
}

int rule110(int phase, int next, int prev){
    int out;

    if(      prev == 1 && phase == 1 && next == 1){
	out = 0;
    }else if(prev == 1 && phase == 1 && next == 0){
	out = 1;
    }else if(prev == 1 && phase == 0 && next == 1){
	out = 1;
    }else if(prev == 1 && phase == 0 && next == 0){
	out = 0;
    }else if(prev == 0 && phase == 1 && next == 1){
	out = 1;
    }else if(prev == 0 && phase == 1 && next == 0){
	out = 1;
    }else if(prev == 0 && phase == 0 && next == 1){
	out = 1;
    }else if(prev == 0 && phase == 0 && next == 0){
	out = 0;
    }
    return out;
}

// SIMULATION

int main (){
    int i;
    int SpaceSize = 100;
    int simtime = 50;
    CellSpace space = CellSpaceCreate(SpaceSize);
    int (*rule)(int,int,int) = &rule110;


    CellSpaceAdd(&space,50,1);
    for(i=0;i<simtime;i++)
	CellSpaceStep(&space,rule);

    CellSpaceDelete(space);
    return 1;
}


#endif
      
 


