/***************************************************************************************************
* Author: 
*        Victorino Sanz 
*        Dpto. Informática y Automática, UNED
*        Juan del Rosal, 16
*        28040, Madrid
*        Spain
*        Email: vsanz@dia.uned.es
*
* Licensed by Victorino Sanz under the LGPL License
* Copyright © 2013, Victorino Sanz.
* 
****************************************************************************************************/

#ifndef CELL2D
#define CELL2D



#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

/* Object data type */

typedef struct Cell2D{
    int phase;
    int x;
    int y;
    int *n;
} Cell2D;

typedef struct CellSpace2D{
  Cell2D **M;
  int cols;
  int rows;
  FILE * gp;
}CellSpace2D;


int mod (int a, int b)
{
   if(b < 0) //you can check for b == 0 separately and do what you want
     return mod(-a, -b);   
   int ret = a % b;
   if(ret < 0)
     ret+=b;
   return ret;
}

/* Object list functions ###################################################*/


CellSpace2D *CellSpace2DCreate(int rows, int columns){
    int i,j,k;
    CellSpace2D *s;
    
    s = (CellSpace2D *)malloc(sizeof(CellSpace2D));
    s->M = (Cell2D **)malloc(columns*sizeof(Cell2D*));    
    for(i=0;i<columns;i++)
	s->M[i] = (Cell2D *)malloc(rows*sizeof(Cell2D));
    s->cols = columns;
    s->rows = rows;
    for(i=0;i<rows;i++)
	for(j=0;j<columns;j++){
	    s->M[i][j].phase = 0; // start dead
	    s->M[i][j].x = i; // position in the space
	    s->M[i][j].y = j;
	    s->M[i][j].n = (int *)malloc(sizeof(int)*8);
	    for(k=0;k<8;k++)
		s->M[i][j].n[k] = 0;
	}
    
    return s;   
}

int CSCreate(int rows, int columns,const char *path){
  CellSpace2D *s;
  
  s = CellSpace2DCreate(rows,columns);
  s->gp = popen(path, "w");
  if(s->gp == NULL){
    fprintf(stderr, "Oops, I can't find %s.", path);
    exit(EXIT_FAILURE);
    }    
  fprintf(s->gp, "set terminal x11 \n");
  
  return (int)s;
  
}

int CellSpace2DDelete(CellSpace2D s){
    free(s.M);
    s.cols = 0;
    s.rows = 0;
    return 1;
}

int printGNUPLOT(int space){
  int k,l;
  CellSpace2D *s;
  
  s = (CellSpace2D *)space;
  fprintf(s->gp,"plot \'-\' matrix with image\n");
  for(k=s->rows-1;k>=0;k--){
    for(l=0;l<s->cols;l++){
      fprintf(s->gp,"%d ",CellSpaceReadPos2D(space,k,l));
    }
    fprintf(s->gp,"\n");
  }
  fprintf(s->gp,"\n");
  fprintf(s->gp,"e\n");
  usleep(50000);
  fflush(s->gp);
 
  return 0;
}

int CellSpace2DAdd(int space, int x,int y, int newphase){
  int i,j,k,l;
    CellSpace2D *s;
    
    s = (CellSpace2D *)space;

    s->M[x][y].phase = newphase;

    // update neighbors (wrapped boundaries)
    s->M[mod((x-1),s->rows)][mod((y-1),s->cols)].n[7] = newphase;
    s->M[mod((x-1),s->rows)][mod(y,s->cols)].n[6] = newphase;
    s->M[mod((x-1),s->rows)][mod((y+1),s->cols)].n[5] = newphase;
    s->M[mod(x,s->rows)][mod((y-1),s->cols)].n[4] = newphase;
    s->M[mod(x,s->rows)][mod((y+1),s->cols)].n[3] = newphase;
    s->M[mod((x+1),s->rows)][mod((y-1),s->cols)].n[2] = newphase;
    s->M[mod((x+1),s->rows)][mod(y,s->cols)].n[1] = newphase;
    s->M[mod((x+1),s->rows)][mod((y+1),s->cols)].n[0] = newphase;
    
    printGNUPLOT(space);
    
    return 1;
}

int CellSpaceReadPos2D(int space, int x,int y){
    CellSpace2D *s;

    //    printf("ReadPOS \n");
    s = (CellSpace2D *)space;
    //printf("X = %d, Y = %d\n",x,y);
    return s->M[x][y].phase;
}


int CellSpace2DStep(int space, int (*rule)(int,int *)){
    int **new;//[s->rows][s->cols];
    int i,j;
    CellSpace2D *s;

    s = (CellSpace2D *)space;
    new = (int**)malloc(sizeof(int *)*s->rows);
    for(i=0;i<s->rows;i++)
      new[i] = (int *)malloc(sizeof(int)*s->cols);

    // calculate and print new phases
    fprintf(s->gp,"plot \'-\' matrix with image\n");
    for(i=s->rows-1;i>=0;i--){
	for(j=0;j<s->cols;j++){
	    new[i][j] = rule(s->M[i][j].phase,s->M[i][j].n);
	    fprintf(s->gp,"%d ",new[i][j]);
	}
	fprintf(s->gp,"\n");
    }
    fprintf(s->gp,"\n");
    fprintf(s->gp,"e\n");
    usleep(50000);
    fflush(s->gp);

    // update neighbor phases
    for(i=0;i<s->rows;i++)
      for(j=0;j<s->cols;j++){
	s->M[i][j].phase = new[i][j];
	s->M[i][j].n[0] = new[mod((i-1),s->rows)][mod((j-1),s->cols)];
	s->M[i][j].n[1] = new[mod((i-1),s->rows)][mod(j,s->cols)];
	s->M[i][j].n[2] = new[mod((i-1),s->rows)][mod((j+1),s->cols)];
	s->M[i][j].n[3] = new[mod(i,s->rows)][mod((j-1),s->cols)];
	s->M[i][j].n[4] = new[mod(i,s->rows)][mod((j+1),s->cols)];
	s->M[i][j].n[5] = new[mod((i+1),s->rows)][mod((j-1),s->cols)];
	s->M[i][j].n[6] = new[mod((i+1),s->rows)][mod(j,s->cols)];
	s->M[i][j].n[7] = new[mod((i+1),s->rows)][mod((j+1),s->cols)];
	
      }

    //printGNUPLOT(space);

    return 1;
}

// RULE FUNCTIONS

int gol(int phase, int *n){
    int sum,i,out;

    sum = 0;
    for(i=0;i<8;i++)
	sum += n[i];

    out = phase;
    if (!phase && sum == 3)
	out = 1;
    else if (phase && (sum < 2 || sum > 3 ) )
	out = 0;
    
    return out;
}

int CSStepGOL(int space){
  int (*rule)(int,int*) = &gol;
  
  CellSpace2DStep(space,rule);
  return 1;
}

// ************************************************************************
// SIMULATION
// ************************************************************************

/* int main (int argc,char** argv){ */
/*   int i,k,l; */
/*     int simtime = 50; */
/*     int space = CSCreate(SSIZE,SSIZE); */
/*     //FILE *data; */
/*     FILE *gp; */

/*     //    Phase = (int **)malloc(sizeof(int *)*SSIZE); */
/*     // for(i=0;i<SSIZE;i++) */
/*     //  Phase[i] = (int *)malloc(sizeof(int)*SSIZE); */
    
/*     gp = popen(GNUPLOT_PATH, "w"); */
/*     if(gp == NULL){ */
/*       fprintf(stderr, "Oops, I can't find %s.", GNUPLOT_PATH); */
/*       exit(EXIT_FAILURE); */
/*     }     */
/*     fprintf(gp, "set terminal x11 \n"); */
/*     // fprintf(gp, "set cbrange [0:1]\n"); */
/*     //fprintf(gp, "unset cbtics\n"); */
/*     //fprintf(gp, "set xrange [0:%d]\n",SSIZE); */
/*     //fprintf(gp, "set yrange [0:%d]\n",SSIZE); */
/*     CellSpace2DAdd(space,0,1,1); */
/*     CellSpace2DAdd(space,1,2,1); */
/*     CellSpace2DAdd(space,2,0,1); */
/*     CellSpace2DAdd(space,2,1,1); */
/*     CellSpace2DAdd(space,2,2,1); */
/*     //data = fopen("cell.dat","w"); */
/*     //for(k=SSIZE-1;k>=0;k--){ */
/*     //  for(l=0;l<SSIZE;l++) */
/*     //	fprintf(data,"%d ",CellSpaceReadPos2D(space,k,l)); */
/*     //  fprintf(data,"\n"); */
/*     //} */
/*     //    fprintf(gp,"plot \"cell.dat\" matrix with image\n"); */
/*     fprintf(gp,"plot \'-\' matrix with image\n"); */
/*     for(k=SSIZE-1;k>=0;k--){ */
/*       for(l=0;l<SSIZE;l++){ */
/* 	fprintf(gp,"%d ",CellSpaceReadPos2D(space,k,l)); */
/*       } */
/*       fprintf(gp,"\n"); */
/*     } */
/*     fprintf(gp,"\n"); */
/*     fprintf(gp,"e\n"); */
/*     usleep(DELAY); */
/*     fflush(gp); */
/*     //fclose(data); */
/*     for(i=0;i<simtime;i++){ */
/*       //data = fopen("cell.dat","w"); */
/*       CSStepGOL(space); */
/*       fprintf(gp,"plot \'-\' matrix with image\n"); */
/*       for(k=SSIZE-1;k>=0;k--){ */
/* 	for(l=0;l<SSIZE;l++){ */
/* 	  fprintf(gp,"%d ",CellSpaceReadPos2D(space,k,l)); */
/* 	} */
/* 	fprintf(gp,"\n"); */
/*       } */
/*       fprintf(gp,"\n"); */
/*       fprintf(gp,"e\n"); */
/*       //for(k=SSIZE-1;k>=0;k--){ */
/*       //for(l=0;l<SSIZE;l++) */
/*       //  fprintf(data,"%d ",CellSpaceReadPos2D(space,k,l)); */
/*       //fprintf(data,"\n"); */
/*       //} */
/*       //fprintf(gp,"plot \"cell.dat\" matrix with image\n"); */
/*       usleep(DELAY); */
/*       fflush(gp); */
/*       //fclose(data); */

/*     } */
 
/*     //getchar(); */
/*     pclose(gp); */
/*     return(EXIT_SUCCESS); */
/* } */


#endif
      
 


